@@ -1,5 +1,5 @@
 # 2ping - A bi-directional ping utility
-# Copyright (C) 2010-2018 Ryan Finnie
+# Copyright (C) 2010-2020 Ryan Finnie
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,4 +16,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-__version__ = '4.3'
+import sys
+
+
+__version__ = "4.5"
+assert sys.version_info > (3, 5)
