@@ -1,5 +1,5 @@
 # 2ping - A bi-directional ping utility
-# Copyright (C) 2010-2018 Ryan Finnie
+# Copyright (C) 2010-2020 Ryan Finnie
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,33 +16,26 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-import random
-import hmac
-import time
-from . import crc32
-from .utils import twoping_checksum, npack, nunpack
-from math import ceil
 import hashlib
+import hmac
+from math import ceil
+import time
 
-try:
-    from Crypto.Cipher import AES
-    has_aes = True
-except ImportError:
-    has_aes = False
+from . import crc32
+from .utils import AES, npack, nunpack, random, twoping_checksum
 
 
-class Extended():
+class Extended:
     id = None
 
     def __init__(self):
-        self.data = b''
+        self.data = b""
 
     def __repr__(self):
         if self.id is None:
-            return '<Extended: {} bytes>'.format(len(self.data))
+            return "<Extended: {} bytes>".format(len(self.data))
         else:
-            id_hex = ''.join(['{:02x}'.format(x) for x in npack(self.id, 4)])
-            return '<Extended (0x{}): {} bytes>'.format(id_hex, len(self.data))
+            return "<Extended (0x{:08x}): {} bytes>".format(self.id, len(self.data))
 
     def load(self, data):
         self.data = data
@@ -56,40 +49,42 @@ class ExtendedText(Extended):
         self.text = str()
 
     def __repr__(self):
-        return '<Text (Generic): {}>'.format(self.text)
+        return "<Text (Generic): {}>".format(self.text)
 
     def load(self, data):
-        self.text = data.decode('UTF-8')
+        self.text = data.decode("UTF-8")
 
     def dump(self, max_length=None):
-        text_bytes = self.text.encode('UTF-8')
+        text_bytes = self.text.encode("UTF-8")
         if (max_length is not None) and (max_length < len(text_bytes)):
             return None
         return text_bytes
 
 
 class ExtendedVersion(ExtendedText):
-    id = 0x3250564e
+    id = 0x3250564E
 
     def __repr__(self):
-        return '<Version: {}>'.format(self.text)
+        return "<Version: {}>".format(self.text)
 
 
 class ExtendedNotice(ExtendedText):
-    id = 0xa837b44e
+    id = 0xA837B44E
 
     def __repr__(self):
-        return '<Notice: {}>'.format(self.text)
+        return "<Notice: {}>".format(self.text)
 
 
 class ExtendedWallClock(Extended):
-    id = 0x64f69319
+    id = 0x64F69319
 
     def __init__(self):
         self.time_us = 0
 
     def __repr__(self):
-        return '<Wall Clock: {}>'.format(time.strftime('%c', time.gmtime(self.time_us / 1000000.0)))
+        return "<Wall Clock: {}>".format(
+            time.strftime("%c", time.gmtime(self.time_us / 1000000.0))
+        )
 
     def load(self, data):
         self.time_us = nunpack(data[0:8])
@@ -101,14 +96,16 @@ class ExtendedWallClock(Extended):
 
 
 class ExtendedMonotonicClock(Extended):
-    id = 0x771d8dfb
+    id = 0x771D8DFB
 
     def __init__(self):
         self.generation = 0
         self.time_us = 0
 
     def __repr__(self):
-        return '<Monotonic Clock: {:0.9f}, gen {}>'.format((self.time_us / 1000000.0), self.generation)
+        return "<Monotonic Clock: {:0.9f}, gen {}>".format(
+            (self.time_us / 1000000.0), self.generation
+        )
 
     def load(self, data):
         self.generation = nunpack(data[0:2])
@@ -121,15 +118,15 @@ class ExtendedMonotonicClock(Extended):
 
 
 class ExtendedRandom(Extended):
-    id = 0x2ff6ad68
+    id = 0x2FF6AD68
 
     def __init__(self):
         self.is_hwrng = False
         self.is_os = False
-        self.random_data = b''
+        self.random_data = b""
 
     def __repr__(self):
-        return '<Random: {} ({}), HWRNG {}, OS {}>'.format(
+        return "<Random: {} ({}), HWRNG {}, OS {}>".format(
             repr(self.random_data),
             len(self.random_data),
             repr(self.is_hwrng),
@@ -150,7 +147,7 @@ class ExtendedRandom(Extended):
             if max_length < 3:
                 return None
             if max_length < (len(random_data) - 2):
-                random_data = random_data[0:max_length-2]
+                random_data = random_data[0 : max_length - 2]
         flags = 0
         if self.is_hwrng:
             flags = flags | 0x0001
@@ -160,16 +157,19 @@ class ExtendedRandom(Extended):
 
 
 class ExtendedBatteryLevels(Extended):
-    id = 0x88a1f7c7
+    id = 0x88A1F7C7
 
     def __init__(self):
         self.batteries = {}
 
     def __repr__(self):
-        return '<Batteries ({}): [{}]>'.format(
+        return "<Batteries ({}): [{}]>".format(
             len(self.batteries),
-            ', '.join(
-                ['{}: {:0.03%}'.format(x, self.batteries[x] / 65535.0) for x in sorted(self.batteries)]
+            ", ".join(
+                [
+                    "{}: {:0.03%}".format(x, self.batteries[x] / 65535.0)
+                    for x in sorted(self.batteries)
+                ]
             ),
         )
 
@@ -177,17 +177,17 @@ class ExtendedBatteryLevels(Extended):
         self.batteries = {}
         pos = 2
         for i in range(nunpack(data[0:2])):
-            battery_id = nunpack(data[pos:pos+2])
-            battery_level = nunpack(data[pos+2:pos+4])
+            battery_id = nunpack(data[pos : pos + 2])
+            battery_level = nunpack(data[pos + 2 : pos + 4])
             self.batteries[battery_id] = battery_level
             pos += 4
 
     def dump(self, max_length=None):
-        if (max_length is not None):
-            if (max_length < 6):
+        if max_length is not None:
+            if max_length < 6:
                 return None
             batteries = {}
-            for i in sorted(self.batteries.keys())[0:int((max_length - 2) / 4)]:
+            for i in sorted(self.batteries.keys())[0 : int((max_length - 2) / 4)]:
                 batteries[i] = self.batteries[i]
         else:
             batteries = self.batteries
@@ -199,18 +199,17 @@ class ExtendedBatteryLevels(Extended):
         return out
 
 
-class Opcode():
+class Opcode:
     id = None
 
     def __init__(self):
-        self.data = b''
+        self.data = b""
 
     def __repr__(self):
         if self.id is None:
-            return '<Opcode: {} bytes>'.format(len(self.data))
+            return "<Opcode: {} bytes>".format(len(self.data))
         else:
-            id_hex = ''.join(['{:02x}'.format(x) for x in npack(self.id, 2)])
-            return '<Opcode (0x{}): {} bytes>'.format(id_hex, len(self.data))
+            return "<Opcode (0x{:04x}): {} bytes>".format(self.id, len(self.data))
 
     def load(self, data):
         self.data = data
@@ -226,24 +225,23 @@ class OpcodeReplyRequested(Opcode):
         pass
 
     def __repr__(self):
-        return '<Reply Requested>'
+        return "<Reply Requested>"
 
     def load(self, data):
         pass
 
     def dump(self, max_length=None):
-        return b''
+        return b""
 
 
 class OpcodeInReplyTo(Opcode):
     id = 0x0002
 
     def __init__(self):
-        self.message_id = b''
+        self.message_id = b""
 
     def __repr__(self):
-        message_id_hex = ''.join(['{:02x}'.format(x) for x in self.message_id])
-        return '<In Reply To: 0x{}>'.format(message_id_hex)
+        return "<In Reply To: 0x{}>".format(self.message_id.hex())
 
     def load(self, data):
         self.message_id = data[0:6]
@@ -261,7 +259,7 @@ class OpcodeRTTEnclosed(Opcode):
         self.rtt_us = 0
 
     def __repr__(self):
-        return '<RTT Enclosed: {} us>'.format(self.rtt_us)
+        return "<RTT Enclosed: {} us>".format(self.rtt_us)
 
     def load(self, data):
         self.rtt_us = nunpack(data[0:4])
@@ -273,25 +271,30 @@ class OpcodeRTTEnclosed(Opcode):
 
 
 class OpcodeMessageIDList(Opcode):
+    _repr_name = "ID List (Generic)"
+
     def __init__(self):
         self.message_ids = []
 
     def __repr__(self):
-        ids = ['0x{}'.format(''.join(['{:02x}'.format(x) for x in y]) for y in self.message_ids)]
-        return '<ID List (Generic): [{}] ({})>'.format(', '.join(ids), len(self.message_ids))
+        return "<{}: [{}] ({})>".format(
+            self._repr_name,
+            ", ".join(["0x{}".format(x.hex()) for x in self.message_ids]),
+            len(self.message_ids),
+        )
 
     def load(self, data):
         self.message_ids = []
         pos = 2
         for i in range(nunpack(data[0:2])):
-            self.message_ids.append(data[pos:pos+6])
+            self.message_ids.append(data[pos : pos + 6])
             pos += 6
 
     def dump(self, max_length=None):
-        if (max_length is not None):
-            if (max_length < 8):
+        if max_length is not None:
+            if max_length < 8:
                 return None
-            output_ids = self.message_ids[0:int((max_length - 2) / 6)]
+            output_ids = self.message_ids[0 : int((max_length - 2) / 6)]
         else:
             output_ids = self.message_ids
 
@@ -303,59 +306,46 @@ class OpcodeMessageIDList(Opcode):
 
 class OpcodeInvestigationSeen(OpcodeMessageIDList):
     id = 0x0008
-
-    def __repr__(self):
-        ids = ['0x{}'.format(''.join(['{:02x}'.format(x) for x in y])) for y in self.message_ids]
-        return '<Investigation Seen: [{}] ({})>'.format(', '.join(ids), len(self.message_ids))
+    _repr_name = "Investigation Seen"
 
 
 class OpcodeInvestigationUnseen(OpcodeMessageIDList):
     id = 0x0010
-
-    def __repr__(self):
-        ids = ['0x{}'.format(''.join(['{:02x}'.format(x) for x in y])) for y in self.message_ids]
-        return '<Investigation Unseen: [{}] ({})>'.format(', '.join(ids), len(self.message_ids))
+    _repr_name = "Investigation Unseen"
 
 
 class OpcodeInvestigate(OpcodeMessageIDList):
     id = 0x0020
-
-    def __repr__(self):
-        ids = ['0x{}'.format(''.join(['{:02x}'.format(x) for x in y])) for y in self.message_ids]
-        return '<Investigate: [{}] ({})>'.format(', '.join(ids), len(self.message_ids))
+    _repr_name = "Investigate"
 
 
 class OpcodeCourtesyExpiration(OpcodeMessageIDList):
     id = 0x0040
-
-    def __repr__(self):
-        ids = ['0x{}'.format(''.join(['{:02x}'.format(x) for x in y])) for y in self.message_ids]
-        return '<Courtesy Expiration: [{}] ({})>'.format(', '.join(ids), len(self.message_ids))
+    _repr_name = "Courtesy Expiration"
 
 
 class OpcodeHMAC(Opcode):
     id = 0x0080
 
     def __init__(self):
-        self.key = b''
+        self.key = b""
         self.digest_index = None
-        self.hash = b''
+        self.hash = b""
 
         self.digest_map = {
-            1: (hashlib.md5, 16, 'HMAC-MD5'),
-            2: (hashlib.sha1, 20, 'HMAC-SHA1'),
-            3: (hashlib.sha256, 32, 'HMAC-SHA256'),
-            4: (crc32, 4, 'HMAC-CRC32'),
-            5: (hashlib.sha512, 64, 'HMAC-SHA512'),
+            1: (hashlib.md5, 16, "HMAC-MD5"),
+            2: (hashlib.sha1, 20, "HMAC-SHA1"),
+            3: (hashlib.sha256, 32, "HMAC-SHA256"),
+            4: (crc32, 4, "HMAC-CRC32"),
+            5: (hashlib.sha512, 64, "HMAC-SHA512"),
         }
 
     def __repr__(self):
         if self.digest_index is not None:
-            return '<{}: 0x{}>'.format(
-                self.digest_map[self.digest_index][2],
-                ''.join(['{:02x}'.format(x) for x in self.hash]),
+            return "<{}: 0x{}>".format(
+                self.digest_map[self.digest_index][2], self.hash.hex()
             )
-        return '<HMAC>'
+        return "<HMAC>"
 
     def load(self, data):
         self.digest_index = nunpack(data[0:2])
@@ -375,7 +365,7 @@ class OpcodeHostLatency(Opcode):
         self.delay_us = 0
 
     def __repr__(self):
-        return '<Host Latency: {} us>'.format(self.delay_us)
+        return "<Host Latency: {} us>".format(self.delay_us)
 
     def load(self, data):
         self.delay_us = nunpack(data[0:4])
@@ -390,25 +380,25 @@ class OpcodeEncrypted(Opcode):
     id = 0x0200
 
     def __init__(self):
-        self.hkdf_info = b'\xd8\x89\xac\x93\xac\xeb\xa1\xf3\x98\xd0\xc6\x9b\xc8\xc6\xa7\xaa'
+        self.hkdf_info = (
+            b"\xd8\x89\xac\x93\xac\xeb\xa1\xf3\x98\xd0\xc6\x9b\xc8\xc6\xa7\xaa"
+        )
         self.method_index = None
-        self.encrypted = b''
-        self.session = b''
+        self.encrypted = b""
+        self.session = b""
         self.iv = None
 
-        self.method_map = {
-            1: ('HKDF-AES256-CBC',),
-        }
+        self.method_map = {1: ("HKDF-AES256-CBC",)}
 
     def __repr__(self):
         if self.method_index is not None:
-            return '<{} (Session {}, IV {}, {} bytes)>'.format(
+            return "<{} (Session {}, IV {}, {} bytes)>".format(
                 self.method_map[self.method_index][0],
                 repr(self.session),
                 repr(self.iv),
                 len(self.encrypted),
             )
-        return '<Encrypted>'
+        return "<Encrypted>"
 
     def load(self, data):
         self.method_index = nunpack(data[0:2])
@@ -425,38 +415,50 @@ class OpcodeEncrypted(Opcode):
         return None
 
     def encrypt(self, unencrypted, key):
-        if not has_aes:
+        if isinstance(AES, ImportError):
             return None
         if self.method_index is None:
             return None
         if self.method_index == 1:
             if self.iv is None:
                 self.iv = bytes([random.randint(0, 255) for x in range(16)])
-            aeskey = self.hkdf(32, key, salt=self.iv, info=(self.hkdf_info + self.session), digestmod=hashlib.sha256)
+            aeskey = self.hkdf(
+                32,
+                key,
+                salt=self.iv,
+                info=(self.hkdf_info + self.session),
+                digestmod=hashlib.sha256,
+            )
             aes_e = AES.new(aeskey, AES.MODE_CBC, self.iv)
             self.encrypted = aes_e.encrypt(unencrypted)
         else:
             return None
 
     def decrypt(self, key):
-        if not has_aes:
+        if isinstance(AES, ImportError):
             return None
         if self.method_index is None:
             return None
         if self.method_index == 1:
-            aeskey = self.hkdf(32, key, salt=self.iv, info=(self.hkdf_info + self.session), digestmod=hashlib.sha256)
+            aeskey = self.hkdf(
+                32,
+                key,
+                salt=self.iv,
+                info=(self.hkdf_info + self.session),
+                digestmod=hashlib.sha256,
+            )
             aes_d = AES.new(aeskey, AES.MODE_CBC, self.iv)
             return aes_d.decrypt(self.encrypted)
 
-    def hkdf(self, length, ikm, salt=b'', info=b'', digestmod=None):
+    def hkdf(self, length, ikm, salt=b"", info=b"", digestmod=None):
         if digestmod is None:
             digestmod = hashlib.sha256
         prk = hmac.new(salt, ikm, digestmod).digest()
         hash_len = len(prk)
-        t = b''
-        okm = b''
+        t = b""
+        okm = b""
         for i in range(ceil(length / hash_len)):
-            t = hmac.new(prk, t + info + bytes([1+i]), digestmod).digest()
+            t = hmac.new(prk, t + info + bytes([1 + i]), digestmod).digest()
             okm += t
         return okm[:length]
 
@@ -469,7 +471,9 @@ class OpcodeExtended(Opcode):
         self.segment_data_positions = {}
 
     def __repr__(self):
-        return '<Extended: {}>'.format(repr(sorted(self.segments.values(), key=lambda x: x.id)))
+        return "<Extended: {}>".format(
+            repr(sorted(self.segments.values(), key=lambda x: x.id))
+        )
 
     def load(self, data):
         self.segments = {}
@@ -486,9 +490,9 @@ class OpcodeExtended(Opcode):
         )
 
         while pos < len(data):
-            flag = nunpack(data[pos:pos+4])
+            flag = nunpack(data[pos : pos + 4])
             pos += 4
-            segment_data_length = nunpack(data[pos:pos+2])
+            segment_data_length = nunpack(data[pos : pos + 2])
             pos += 2
             self.segment_data_positions[flag] = (pos, segment_data_length)
             segment_handler = None
@@ -500,13 +504,13 @@ class OpcodeExtended(Opcode):
                 segment_handler = Extended
                 segment_handler.id = flag
             self.segments[flag] = segment_handler()
-            self.segments[flag].load(data[pos:(pos+segment_data_length)])
+            self.segments[flag].load(data[pos : (pos + segment_data_length)])
             pos += segment_data_length
 
     def dump(self, max_length=None):
         if (max_length is not None) and (max_length < 6):
             return None
-        out = b''
+        out = b""
         pos = 0
         for segment in self.segments.values():
             if max_length is None:
@@ -527,30 +531,30 @@ class OpcodeExtended(Opcode):
         return out
 
 
-class Packet():
+class Packet:
     def __repr__(self):
-        return '<Packet (0x{}): {}>'.format(
-            ''.join(['{:02x}'.format(x) for x in self.message_id]),
-            repr(sorted(self.opcodes.values(), key=lambda x: x.id))
+        return "<Packet (0x{}): {}>".format(
+            self.message_id.hex(),
+            repr(sorted(self.opcodes.values(), key=lambda x: x.id)),
         )
 
     def __init__(self):
-        self.message_id = b''
+        self.message_id = b""
         self.opcodes = {}
         self.min_length = 0
         self.max_length = 1024
         self.align_length = 0
-        self.padding_pattern = b'\x00'
+        self.padding_pattern = b"\x00"
         self.opcode_data_positions = {}
 
     def load(self, data):
         magic_number = data[0:2]
-        if magic_number != b'\x32\x50':
-            raise Exception('Invalid magic number')
+        if magic_number != b"\x32\x50":
+            raise Exception("Invalid magic number")
         checksum = nunpack(data[2:4])
         if checksum:
-            if twoping_checksum(data[0:2] + b'\x00\x00' + data[4:]) != checksum:
-                raise Exception('Invalid checksum')
+            if twoping_checksum(data[0:2] + b"\x00\x00" + data[4:]) != checksum:
+                raise Exception("Invalid checksum")
         self.message_id = data[4:10]
         opcode_flags = nunpack(data[10:12])
         self.opcodes = {}
@@ -572,7 +576,7 @@ class Packet():
         for flag in (2 ** x for x in range(16)):
             if not opcode_flags & flag:
                 continue
-            opcode_data_length = nunpack(data[pos:pos+2])
+            opcode_data_length = nunpack(data[pos : pos + 2])
             pos += 2
             self.opcode_data_positions[flag] = (pos, opcode_data_length)
             opcode_handler = None
@@ -584,7 +588,7 @@ class Packet():
                 opcode_handler = Opcode
                 opcode_handler.id = flag
             self.opcodes[flag] = opcode_handler()
-            self.opcodes[flag].load(data[pos:(pos+opcode_data_length)])
+            self.opcodes[flag].load(data[pos : (pos + opcode_data_length)])
             pos += opcode_data_length
 
     def dump(self):
@@ -611,14 +615,16 @@ class Packet():
                 continue
             if (packet_length + 2) > self.max_length:
                 break
-            res = self.opcodes[flag].dump(max_length=(self.max_length - packet_length - 2))
+            res = self.opcodes[flag].dump(
+                max_length=(self.max_length - packet_length - 2)
+            )
             if res is None:
                 continue
             opcode_datas[flag] = res
             res_len = len(res)
             packet_length += res_len + 2
         opcode_flags = 0
-        opcode_data = b''
+        opcode_data = b""
         packet_length = 12
         for flag in sorted(opcode_datas.keys()):
             res = opcode_datas[flag]
@@ -630,7 +636,9 @@ class Packet():
             opcode_data += npack(res_len, 2)
             opcode_data += res
             packet_length += res_len + 2
-        out = bytearray(b'\x32\x50\x00\x00' + self.message_id + npack(opcode_flags, 2) + opcode_data)
+        out = bytearray(
+            b"\x32\x50\x00\x00" + self.message_id + npack(opcode_flags, 2) + opcode_data
+        )
         target_length = len(out)
         if len(out) < self.min_length:
             target_length = self.min_length
@@ -638,10 +646,15 @@ class Packet():
             target_length += self.align_length - (target_length % self.align_length)
         if len(out) < target_length:
             target_padding = target_length - len(out)
-            padding = (self.padding_pattern * int(target_padding / len(self.padding_pattern) + 1))[0:target_padding]
+            padding = (
+                self.padding_pattern
+                * int(target_padding / len(self.padding_pattern) + 1)
+            )[0:target_padding]
             out += padding
         if (OpcodeHMAC.id in self.opcodes) and auth_pos_begin:
-            out[auth_pos_begin:auth_pos_end] = self.calculate_hash(self.opcodes[OpcodeHMAC.id], out)
+            out[auth_pos_begin:auth_pos_end] = self.calculate_hash(
+                self.opcodes[OpcodeHMAC.id], out
+            )
         out[2:4] = npack(twoping_checksum(out), 2)
         return bytes(out)
 
